
#ifndef FYGUI_EXPORT_H
#define FYGUI_EXPORT_H

#ifdef FYGUI_STATIC_DEFINE
#  define FYGUI_EXPORT
#  define FYGUI_NO_EXPORT
#else
#  ifndef FYGUI_EXPORT
#    ifdef fooyin_gui_EXPORTS
        /* We are building this library */
#      define FYGUI_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define FYGUI_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef FYGUI_NO_EXPORT
#    define FYGUI_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef FYGUI_DEPRECATED
#  define FYGUI_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef FYGUI_DEPRECATED_EXPORT
#  define FYGUI_DEPRECATED_EXPORT FYGUI_EXPORT FYGUI_DEPRECATED
#endif

#ifndef FYGUI_DEPRECATED_NO_EXPORT
#  define FYGUI_DEPRECATED_NO_EXPORT FYGUI_NO_EXPORT FYGUI_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef FYGUI_NO_DEPRECATED
#    define FYGUI_NO_DEPRECATED
#  endif
#endif

#endif /* FYGUI_EXPORT_H */
