
#ifndef FYUTILS_EXPORT_H
#define FYUTILS_EXPORT_H

#ifdef FYUTILS_STATIC_DEFINE
#  define FYUTILS_EXPORT
#  define FYUTILS_NO_EXPORT
#else
#  ifndef FYUTILS_EXPORT
#    ifdef fooyin_utils_EXPORTS
        /* We are building this library */
#      define FYUTILS_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define FYUTILS_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef FYUTILS_NO_EXPORT
#    define FYUTILS_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef FYUTILS_DEPRECATED
#  define FYUTILS_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef FYUTILS_DEPRECATED_EXPORT
#  define FYUTILS_DEPRECATED_EXPORT FYUTILS_EXPORT FYUTILS_DEPRECATED
#endif

#ifndef FYUTILS_DEPRECATED_NO_EXPORT
#  define FYUTILS_DEPRECATED_NO_EXPORT FYUTILS_NO_EXPORT FYUTILS_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef FYUTILS_NO_DEPRECATED
#    define FYUTILS_NO_DEPRECATED
#  endif
#endif

#endif /* FYUTILS_EXPORT_H */
