
#ifndef FYCORE_EXPORT_H
#define FYCORE_EXPORT_H

#ifdef FYCORE_STATIC_DEFINE
#  define FYCORE_EXPORT
#  define FYCORE_NO_EXPORT
#else
#  ifndef FYCORE_EXPORT
#    ifdef fooyin_core_EXPORTS
        /* We are building this library */
#      define FYCORE_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define FYCORE_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef FYCORE_NO_EXPORT
#    define FYCORE_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef FYCORE_DEPRECATED
#  define FYCORE_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef FYCORE_DEPRECATED_EXPORT
#  define FYCORE_DEPRECATED_EXPORT FYCORE_EXPORT FYCORE_DEPRECATED
#endif

#ifndef FYCORE_DEPRECATED_NO_EXPORT
#  define FYCORE_DEPRECATED_NO_EXPORT FYCORE_NO_EXPORT FYCORE_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef FYCORE_NO_DEPRECATED
#    define FYCORE_NO_DEPRECATED
#  endif
#endif

#endif /* FYCORE_EXPORT_H */
